SystemOrganization addCategory: #'Pier-Book-Structures'!
SystemOrganization addCategory: #'Pier-Book-Utilities'!
SystemOrganization addCategory: #'Pier-Book-Latex'!
SystemOrganization addCategory: #'Pier-Book-Deprecated'!

PRRawView subclass: #BOBookView
	instanceVariableNames: 'book'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOBookView class methodsFor: 'testing' stamp: 'lr 6/11/2009 09:28'!
isAbstract
	^ self name = #BOBookView! !

!BOBookView class methodsFor: 'testing' stamp: 'lr 5/19/2009 11:28'!
isValidIn: aContext
	^ (super isValidIn: aContext) and: [ aContext structure isPublication ]! !

!BOBookView methodsFor: 'accessing' stamp: 'lr 5/19/2009 13:48'!
book
	^ book ifNil: [ book := self findBook ]! !

!BOBookView methodsFor: 'private' stamp: 'lr 5/19/2009 13:48'!
findBook
	^ self context structure findAncestor: [ :each | each isPublication and: [ each isBook ] ]! !

!BOBookView methodsFor: 'accessing' stamp: 'lr 5/19/2009 11:37'!
writer
	^ BOLatexWriter! !

BOBookView subclass: #BOLatexView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOLatexView class methodsFor: 'accessing' stamp: 'lr 11/3/2009 20:32'!
label
	^ 'LaTeX'! !

!BOLatexView methodsFor: 'rendering' stamp: 'lr 11/3/2009 20:44'!
renderContentOn: aResponse
	aResponse 
		contentType: 'text/latex';
		attachmentWithFileName: self book name , '.tex'.
	self writer
		write: self book
		to: aResponse stream! !

BOBookView subclass: #BOPdfView
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOPdfView class methodsFor: 'accessing' stamp: 'lr 5/19/2009 11:25'!
label
	^ 'PDF'! !

!BOPdfView methodsFor: 'configuration' stamp: 'lr 5/19/2009 13:46'!
bibtex
	^ self findExecutable: 'bibtex'! !

!BOPdfView methodsFor: 'accessing' stamp: 'lr 5/19/2009 11:27'!
directory
	^ directory ifNil: [ directory := (FileDirectory default directoryNamed: 'pdf/' , DateAndTime now asSortableString) assureExistence ]! !

!BOPdfView methodsFor: 'accessing' stamp: 'lr 5/19/2009 11:53'!
filename
	^ self book name! !

!BOPdfView methodsFor: 'private' stamp: 'lr 5/19/2009 13:46'!
findExecutable: aString
	| path |
	path := #('/usr/bin' '/usr/local/bin' '/usr/texbin')
		detect: [ :each | (FileDirectory on: each) fileExists: aString ]
		ifNone: [ self error: 'The command ' , aString printString , ' cannot be found.' ].
	^ (FileDirectory on: path) fullNameFor: aString! !

!BOPdfView methodsFor: 'configuration' stamp: 'lr 5/19/2009 13:47'!
intex
	^ self findExecutable: 'makeindex'! !

!BOPdfView methodsFor: 'configuration' stamp: 'lr 5/19/2009 13:47'!
latex
	^ self findExecutable: 'pdflatex'! !

!BOPdfView methodsFor: 'accessing' stamp: 'lr 5/19/2009 13:38'!
process
	^ (Smalltalk classNamed: #OSProcess) ifNil: [ self error: 'OSProcess not found' ]! !

!BOPdfView methodsFor: 'rendering' stamp: 'lr 11/3/2009 20:43'!
renderContentOn: aResponse
	aResponse 
		contentType: 'application/pdf';
		attachmentWithFileName: self book name , '.pdf'.
	self renderPdfOn: aResponse stream! !

!BOPdfView methodsFor: 'rendering' stamp: 'lr 9/10/2009 17:24'!
renderPdfOn: aStream
	| fullname stream latexCommand bibtexCommand indexCommand |
	fullname := self directory fullNameFor: self filename.
	[ stream := self directory forceNewFileNamed: self filename , '.tex'.
	[ stream nextPutAll: (self writer write: self book) ]
		ensure: [ stream close ].
	latexCommand := self latex , ' -output-directory ' , self directory fullName printString , ' -interaction batchmode ' , (fullname , '.tex') printString.
	bibtexCommand := self bibtex , ' -output-directory ' , self directory fullName printString , ' -interaction batchmode ' , (fullname , '.tex') printString.
	indexCommand := self intex , ' -output-directory ' , self directory fullName printString , ' -interaction batchmode ' , (fullname , '.tex') printString.
	self process
		waitForCommand: latexCommand;
		waitForCommand: bibtexCommand;
		waitForCommand: indexCommand;
		waitForCommand: latexCommand;
		waitForCommand: latexCommand.
	stream := self directory oldFileNamed: self filename , '.pdf'.
	^ [ self streamFrom: stream to: aStream ]
		ensure: [ stream close ] ]
			ensure: [ self directory recursiveDelete. directory := nil ]! !

!BOPdfView methodsFor: 'private' stamp: 'lr 5/19/2009 13:48'!
streamFrom: aSourceStream to: aTargetStream
	aSourceStream binary. 
	aTargetStream binary.
	[ aSourceStream atEnd ] 
		whileFalse: [ aTargetStream nextPutAll: (aSourceStream next: 1024) ]! !

BOBookView subclass: #BOZipView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOZipView class methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:47'!
label
	^ 'ZIP'! !

!BOZipView methodsFor: 'adding' stamp: 'lr 11/3/2009 21:01'!
addFilesTo: anArchive
	self book enumerator with all do: [ :structure |
		structure outgoingReferences do: [ :link |
			(link isInternal and: [ link isBroken not and: [ link isEmbedded and: [ link target isFigure ] ] ]) ifTrue: [
				anArchive
					addFile: (link target file directory fullNameFor: link target file filename)
					as: (link target file location reduce: [ :a :b | a , '/' , b ]) , '/' , link target file filename ] ] ]! !

!BOZipView methodsFor: 'adding' stamp: 'lr 11/3/2009 20:21'!
addLatexTo: anArchive
	| contents |
	contents := (self writer write: self book)
		copyReplaceAll: MAExternalFileModel baseDirectory fullName , FileDirectory slash
		with: String new.
	anArchive
		addString: contents
		as: self book name , '.tex'! !

!BOZipView methodsFor: 'rendering' stamp: 'lr 11/3/2009 20:44'!
renderContentOn: aResponse
	| archive |
	aResponse 
		contentType: 'application/zip';
		attachmentWithFileName: self book name , '.zip'.
	archive := ZipArchive new.
	self addLatexTo: archive; addFilesTo: archive.
	archive writeTo: aResponse stream.
	archive close! !

PRStructure subclass: #BONote
	instanceVariableNames: 'email note timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Structures'!

!BONote class methodsFor: 'testing' stamp: 'lr 5/3/2009 12:45'!
canBeChildOf: aStructureClass
	^ aStructureClass includesBehavior: BOPublication! !

!BONote class methodsFor: 'testing' stamp: 'lr 11/16/2009 00:12'!
canBeParentOf: aStructureClass
	^ aStructureClass includesBehavior: self! !

!BONote class methodsFor: 'descriptions' stamp: 'lr 5/3/2009 13:05'!
descriptionConfirmation
	^ MABooleanDescription new
		addCondition: (MACondition selector: #yourself)
			labelled: 'Please, confirm submission by clicking the checkbox';
		checkboxLabel: 'Confirm submission by clicking the checkbox';
		propertyAt: #public put: true;
		accessor: #confirmation;
		label: 'Confirmation';
		priority: 600;
		beEditable;
		yourself! !

!BONote class methodsFor: 'descriptions' stamp: 'lr 9/11/2009 10:50'!
descriptionDocument
	^ PRDocumentDescription new
		accessor: #document;
		label: 'Contents';
		priority: 600;
		beReadonly;
		beEditable;
		beHidden;
		yourself! !

!BONote class methodsFor: 'descriptions' stamp: 'lr 9/11/2009 10:48'!
descriptionEmail
	^ MAStringDescription new
		addCondition: (MACondition selector: #matches: argument: '#*@#*.#*')
			labelled: 'Please make sure to enter a valid E-Mail';
	 	propertyAt: #public put: true;
		accessor: #email;
		label: 'E-Mail';
		priority: 500;
		beRequired;
		beEditable;
		yourself! !

!BONote class methodsFor: 'descriptions' stamp: 'lr 5/3/2009 13:05'!
descriptionNote
	^ MAMemoDescription new
	 	propertyAt: #public put: true;
		accessor: #note;
		priority: 550;
		label: 'Note';
		lineCount: 5;
		beRequired;
		beEditable;
		yourself! !

!BONote class methodsFor: 'testing' stamp: 'lr 5/3/2009 12:43'!
isAbstract
	^ false! !

!BONote class methodsFor: 'accessing' stamp: 'lr 11/16/2009 00:11'!
label
	^ 'Book Note'! !

!BONote methodsFor: 'accessing-dynamic' stamp: 'lr 5/3/2009 12:24'!
author
	^ self email copyUpTo: $@! !

!BONote methodsFor: 'accessing' stamp: 'lr 5/3/2009 13:05'!
confirmation
	^ false! !

!BONote methodsFor: 'accessing' stamp: 'lr 5/3/2009 13:05'!
confirmation: aBoolean! !

!BONote methodsFor: 'accessing-dynamic' stamp: 'lr 9/11/2009 10:18'!
document
	^ PRDocument new
		add: (PRParagraph with: (PRText content: self author));
		add: (PRPreformatted withAll: (self note lines
			collect: [ :each | PRText content: each ]));
		yourself! !

!BONote methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:24'!
email
	^ email! !

!BONote methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:24'!
email: aString 
	email := aString! !

!BONote methodsFor: 'initialization' stamp: 'lr 5/3/2009 12:29'!
initialize
	super initialize.
	email := String new.
	note := String new.
	timestamp := TimeStamp now! !

!BONote methodsFor: 'testing' stamp: 'lr 5/3/2009 12:10'!
isNote
	^ true! !

!BONote methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:09'!
note
	^ note! !

!BONote methodsFor: 'accessing' stamp: 'lr 9/12/2009 16:23'!
note: aString
	note := aString.
	note isNil ifTrue: [ ^ self ].
	note := ((note trimBlanks
		copyReplaceAll: String crlf with: String cr)
		copyReplaceAll: String lf with: '')
		reject: [ :each | each asciiValue > 255 ]! !

!BONote methodsFor: 'accessing-dynamic' stamp: 'lr 5/3/2009 12:29'!
timestamp
	^ timestamp! !

!PRStructure methodsFor: '*pier-book' stamp: 'lr 6/3/2009 22:44'!
isFigure
	^ false! !

!PRStructure methodsFor: '*pier-book' stamp: 'lr 5/3/2009 12:10'!
isNote
	^ false! !

!PRStructure methodsFor: '*pier-book' stamp: 'lr 4/24/2009 11:56'!
isPublication
	^ false! !

PRLink subclass: #BOIndexLink
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Utilities'!

!BOIndexLink class methodsFor: 'testing' stamp: 'lr 5/1/2009 22:08'!
isAbstract
	^ false! !

!BOIndexLink class methodsFor: 'testing' stamp: 'lr 5/1/2009 22:09'!
isValidReference: aString
	^ aString beginsWith: 'index:'! !

!BOIndexLink methodsFor: 'visiting' stamp: 'lr 5/1/2009 22:09'!
accept: aVisitor
	aVisitor visitIndexLink: self! !

!BOIndexLink methodsFor: 'accessing' stamp: 'lr 5/19/2009 11:41'!
entries
	^ self title findTokens: $!!! !

!BOIndexLink methodsFor: 'accessing' stamp: 'lr 5/1/2009 22:13'!
title
	^ super title copyAfter: $:! !

!PRWikiWriter methodsFor: '*pier-book-model' stamp: 'lr 6/19/2009 16:31'!
visitReferenceLink: aLink
	self nextPutAll: aLink class markup; nextPutAll: 'ref:'; nextPutAll: aLink reference; nextPutAll: aLink class markup! !

!PRReferenceRenderer methodsFor: '*pier-book-model' stamp: 'lr 6/19/2009 16:47'!
visitReferenceLink: aLink
	aLink target isFigure ifTrue: [
		^ self visit: (PRInternalLink new
			anchor: aLink target name;
			target: aLink target parent;
			add: (PRText content: aLink title);
			yourself) ].
	aLink target isPublication ifTrue: [
		^ self visit: (PRInternalLink new
			target: aLink target;
			add: (PRText content: aLink title);
			yourself) ].
	^ super visitReferenceLink: aLink! !

PRCase subclass: #BOPublication
	instanceVariableNames: 'numberedTitle ignoredTitle ignoredBody'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Structures'!

BOPublication subclass: #BOBook
	instanceVariableNames: 'subtitle tagline authors additionalAuthors latexHeader latexFooter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Structures'!

!BOBook class methodsFor: 'testing' stamp: 'lr 11/16/2009 08:12'!
canBeChildOf: aStructureClass
	^ (aStructureClass includesBehavior: BOPublication) not! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 12/21/2009 17:21'!
descriptionAdditionalAuthors
	^ MAStringDescription new
		parameterName: 'additionalAuthors';
		accessor: #additionalAuthors;
		label: 'Additional Authors';
		priority: 500;
		beSearchable;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 4/23/2009 19:38'!
descriptionAuthors
	^ MAStringDescription new
		parameterName: 'authors';
		accessor: #authors;
		label: 'Authors';
		beSearchable;
		priority: 400;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 12/30/2009 13:43'!
descriptionLatexFooter
	^ MAMemoDescription new
		comment: 'Everything that appears in the backmatter of the book, before the document ends.';
		accessor: #latexFooter;
		label: 'Footer';
		priority: 1100;
		group: 'LaTeX';
		lineCount: 5;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 12/30/2009 13:42'!
descriptionLatexHeader
	^ MAMemoDescription new
		comment: 'Everythign that appears at the top of the generated code, before the document begins.';
		accessor: #latexHeader;
		label: 'Header';
		priority: 1000;
		group: 'LaTeX';
		lineCount: 10;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 11/16/2009 11:07'!
descriptionNumberedTitle
	^ nil! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 4/23/2009 19:37'!
descriptionSubtitle
	^ MAStringDescription new
		parameterName: 'subtitle';
		accessor: #subtitle;
		label: 'Subtitle';
		beSearchable;
		priority: 200;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'descriptions' stamp: 'lr 4/23/2009 19:37'!
descriptionTagline
	^ MAStringDescription new
		parameterName: 'tagline';
		accessor: #tagline;
		label: 'Tagline';
		beSearchable;
		priority: 300;
		beEditable;
		yourself! !

!BOBook class methodsFor: 'testing' stamp: 'lr 11/16/2009 08:14'!
isAbstract
	^ false! !

!BOBook class methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:31'!
label
	^ 'Book'! !

!BOBook methodsFor: 'visiting' stamp: 'lr 5/13/2009 11:30'!
accept: aVisitor
	aVisitor visitBook: self! !

!BOBook methodsFor: 'accessing' stamp: 'lr 12/21/2009 17:20'!
additionalAuthors
	^ additionalAuthors! !

!BOBook methodsFor: 'accessing' stamp: 'lr 12/21/2009 17:20'!
additionalAuthors: aString
	additionalAuthors := aString! !

!BOBook methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:38'!
authors
	^ authors! !

!BOBook methodsFor: 'accessing' stamp: 'lr 12/21/2009 17:20'!
authors: aString
	authors := aString! !

!BOBook methodsFor: 'defaults' stamp: 'lr 12/30/2009 13:37'!
defaultLatexFooter
	^ '{\small\raggedright\printindex}
\cleardoublepage
\cleardoublepage
\cleardoublepage'	! !

!BOBook methodsFor: 'defaults' stamp: 'lr 1/16/2010 11:26'!
defaultLatexHeader
	^ '\documentclass[a4paper,10pt,twoside]{book}
\usepackage[
	papersize={6.13in,9.21in},
	hmargin={.75in,.75in},
	vmargin={.75in,1in},
	ignoreheadfoot
]{geometry}

% packages
\usepackage{color}
\usepackage{xcolor}
\usepackage{xspace}
\usepackage{ifthen}
\usepackage{amsbsy}
\usepackage{amssymb}
\usepackage{tabularx}
\usepackage{microtype}
\usepackage[english]{babel}
\usepackage[parfill]{parskip}
\usepackage[normalem]{ulem}

% fonts
\usepackage{lmodern}
\usepackage{palatino}
\usepackage[T1]{fontenc}

% references
\usepackage{hyperref}
\usepackage[all]{hypcap}
\setcounter{tocdepth}{2}
\urlstyle{sf}
\hypersetup{
	a4paper,
	colorlinks=true,
	linkcolor=black,
	anchorcolor=black,
	citecolor=black,
	filecolor=black,
	pagecolor=black,
	urlcolor=black,
	plainpages=black,
	bookmarksopen=true,
	bookmarksopenlevel=2}
	
% source code
\usepackage{textcomp}
\usepackage{listings}
\definecolor{source}{gray}{0.95}
\lstset{
	language={},
	% characters
	tabsize=3,
	upquote=true,
	keepspaces=true,
	breaklines=true,
	alsoletter={\#:},
	mathescape=false,
	columns=fullflexible,
	breakautoindent=true,
	showstringspaces=false,
	backgroundcolor=\color{source},
	basicstyle=\footnotesize\sffamily}
\newcommand{\ct}[1]{{\footnotesize\sffamily #1}}
	
% float placement: http://www.tex.ac.uk/cgi-bin/texfaq2html?label=floats
\renewcommand{\topfraction}{.85}
\renewcommand{\bottomfraction}{.7}
\renewcommand{\textfraction}{.15}
\renewcommand{\floatpagefraction}{.66}
\renewcommand{\dbltopfraction}{.66}
\renewcommand{\dblfloatpagefraction}{.66}
\setcounter{topnumber}{9}
\setcounter{bottomnumber}{9}
\setcounter{totalnumber}{20}
\setcounter{dbltopnumber}{9}

% environments
\newlength\importantwidth
\def\findlength{\setlength\importantwidth\linewidth
	\addtolength\importantwidth{-1\fboxrule}
	\addtolength\importantwidth{-1\fboxsep}}
\newenvironment{important}%
	{\baselineskip12pt plus0pt minus 0pt\par\begingroup\setlength{\fboxsep}{5pt}\findlength\setbox0=\vbox\bgroup\hsize=\importantwidth\begin{minipage}{\importantwidth}}%
	{\end{minipage}\egroup{\fboxsep2.5pt\fbox{\fboxsep5pt\box0}}\endgroup\par\ignorespacesafterend}

\newenvironment{seaside28}%
	{\begin{minipage}[t]{0.2\textwidth}\hfill\textcolor{orange}{Seaside 2.8}\end{minipage}\hspace{0.05\textwidth}\begin{minipage}[t]{0.75\textwidth}}%
	{\end{minipage}}
\newenvironment{seaside30}%
	{\begin{minipage}[t]{0.2\textwidth}\hfill\textcolor{orange}{Seaside 3.0}\end{minipage}\hspace{0.05\textwidth}\begin{minipage}[t]{0.75\textwidth}}%
	{\end{minipage}}
\newenvironment{note}% 
	{\begin{minipage}[t]{0.2\textwidth}\hfill\textcolor{gray}{Note}\end{minipage}\hspace{0.05\textwidth}\begin{minipage}[t]{0.75\textwidth}}%
	{\end{minipage}}
\newenvironment{advanced}% 
	{\begin{minipage}[t]{0.2\textwidth}\hfill\textcolor{blue}{Advanced}\end{minipage}\hspace{0.05\textwidth}\begin{minipage}[t]{0.75\textwidth}}%
	{\end{minipage}}
\newenvironment{todo}% 
	{\comment}%
	{\endcomment}
\newenvironment{blockquote}%
	{\begin{quote}}
	{\end{quote}}

% keyword index 
\usepackage{makeidx}
\makeindex
\makeglossary

% graphics
\usepackage{graphicx}
\DeclareGraphicsExtensions{.png,.jpg,.pdf,.gif}

% references
\def\partautorefname{Part}
\def\chapterautorefname{Chapter}
\def\sectionautorefname{Section}
\def\subsectionautorefname{Section}
\def\figureautorefname{Figure}
\def\tableautorefname{Table}'! !

!BOBook methodsFor: 'testing' stamp: 'lr 6/4/2009 09:20'!
isNumberedTitle
	^ false! !

!BOBook methodsFor: 'accessing-latex' stamp: 'lr 12/30/2009 13:38'!
latexFooter
	^ latexFooter ifNil: [ self defaultLatexFooter ]! !

!BOBook methodsFor: 'accessing-latex' stamp: 'lr 12/30/2009 13:38'!
latexFooter: aString
	latexFooter := aString! !

!BOBook methodsFor: 'accessing-latex' stamp: 'lr 12/30/2009 13:32'!
latexHeader
	^ latexHeader ifNil: [ self defaultLatexHeader ]! !

!BOBook methodsFor: 'accessing-latex' stamp: 'lr 12/30/2009 13:31'!
latexHeader: aString
	latexHeader := aString! !

!BOBook methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:38'!
subtitle
	^ subtitle! !

!BOBook methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:38'!
subtitle: aString
	subtitle := aString! !

!BOBook methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:38'!
tagline
	^ tagline! !

!BOBook methodsFor: 'accessing' stamp: 'lr 4/23/2009 19:38'!
tagline: aString
	tagline := aString! !

!BOBook methodsFor: 'accessing' stamp: 'lr 11/16/2009 00:20'!
type
	^ #book! !

BOPublication subclass: #BOChapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Deprecated'!

BOPublication subclass: #BOPart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Deprecated'!

BOPublication subclass: #BOPortion
	instanceVariableNames: 'type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Structures'!

!BOPortion class methodsFor: 'testing' stamp: 'lr 11/16/2009 08:15'!
canBeChildOf: aStructureClass
	^ aStructureClass includesBehavior: BOPublication! !

!BOPortion class methodsFor: 'descriptions' stamp: 'lr 11/16/2009 09:16'!
descriptionLabel
	^ MAStringDescription new
		parameterName: 'label';
		accessor: #label;		
		priority: 50;		
		beReadonly;
		beEditable;
		yourself! !

!BOPortion class methodsFor: 'descriptions' stamp: 'lr 11/16/2009 08:26'!
descriptionNumber
	^ MAStringDescription new
		parameterName: 'number';
		accessor: #number;
		beReadonly;
		yourself! !

!BOPortion class methodsFor: 'initialization' stamp: 'lr 11/16/2009 00:08'!
initialize
	self migrateClasses! !

!BOPortion class methodsFor: 'testing' stamp: 'lr 11/16/2009 08:13'!
isAbstract
	^ false! !

!BOPortion class methodsFor: 'accessing' stamp: 'lr 11/16/2009 08:13'!
label
	^ 'Book Portion'! !

!BOPortion class methodsFor: 'initialization' stamp: 'lr 11/16/2009 11:32'!
migrateClasses
	PRCompatibility allSubInstancesOf: BOSubsection do: [ :structure |
		structure become: (self basicNew
			copyFrom: structure;
			type: #subsection) ].
	PRCompatibility allSubInstancesOf: BOSection do: [ :structure |
		structure become: (self basicNew
			copyFrom: structure;
			type: #section) ].
	PRCompatibility allSubInstancesOf: BOChapter do: [ :structure |
		structure become: (self basicNew
			copyFrom: structure;
			type: #chapter) ].
	PRCompatibility allSubInstancesOf: BOPart do: [ :structure |
		structure become: (self basicNew
			copyFrom: structure;
			type: #part) ].
	PRCompatibility allSubInstancesOf: BOPublication do: [ :structure |
		structure parent: structure parent ]! !

!BOPortion methodsFor: 'visiting' stamp: 'lr 11/16/2009 08:07'!
accept: aVisitor
	aVisitor visitPortion: self! !

!BOPortion methodsFor: 'private' stamp: 'lr 11/16/2009 08:00'!
arabicToRoman: anInteger
	"Convert an arabic integer to a roman string."
	
	| number roman |
	number := anInteger.
	roman := WriteStream on: String new.
	#((1000 'M') (900 'CM') (500 'D') (400 'CD') (100 'C') (90 'XC') (50 'L') (40 'XL') (10 'X') (9 'IX') (5 'V') (4 'IV') (1 'I')) do: [ :pair |
		[ number >= pair first ] whileTrue: [
			roman nextPutAll: pair second.
			number := number - pair first ] ].
	^ roman contents! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 12/20/2009 14:38'!
cssClass
	^ super cssClass ,  ' ' , self type asString! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 10:43'!
description
	"Direct descendents of a book can be a part or a chapter, that's why we needs an additional setting."

	^ (self parent isNil or: [ self parent isBook not ]) 
		ifTrue: [ super description ]
		ifFalse: [
			super description copy
				add: self descriptionType;
				yourself ]! !

!BOPortion methodsFor: 'private' stamp: 'lr 11/16/2009 10:43'!
descriptionType
	^ MASingleOptionDescription new
		optionsAndLabels: (self class typeSymbolsAndLabels
			copyFrom: 2 to: 3);
		accessor: #type;
		priority: 50;
		label: 'Type';
		beEditable;
		beRequired;
		yourself! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 10:45'!
index
	"Answer the index number of this publication, for example the chapter or section number."
	
	| enumerator siblings |
	self parent ifNil: [ ^ nil ].
	enumerator := self isChapter
		ifFalse: [ self parent enumerator ]
		ifTrue: [
			| root |
			root := self
				findAncestor: [ :each | each isPublication and: [ each isBook ] ]
				ifAbsent: [ ^ nil ].
			root enumerator everything ].
	siblings := enumerator select: [ :each | 
		each isPublication 
			and: [ each isNumberedTitle
			and: [ each type = self type ] ] ].
	^ siblings indexOf: self ifAbsent: [ nil ]! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 09:18'!
label
	^ super label , ' ' , (self number ifNil: [ '' ])! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:24'!
number
	"Answer the composed numbers."

	| root |
	self isParagraph
		ifTrue: [ ^ nil ].
	self isPart
		ifTrue: [ ^ self arabicToRoman: (self index ifNil: [ ^ nil ]) ].
	root := self 
		findAncestor: [ :each | each isPublication and: [ each isBook or: [ each isPart ] ] ]
		ifAbsent: [ ^ nil ].
	^ ((((self parents copyAfter: root) collect: [ :each | each index ])
		reject: [ :each | each isNil ]) collect: [ :each | each asString ])
		reduce: [ :a :b | a , '.' , b ]! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:30'!
parent: aStructure
	"This is pretty ugly code that ensures a consistent labeling of book, part, chapter, section, subsection, subsubsection and paragraph. It is complicated because parts are optional. If this overflows just make a paragraph out of everything."

	super parent: aStructure.
	self parent ifNil: [ ^ self ].
	self type: (self parent isBook
		ifTrue: [
			(self isPart or: [ self isChapter ])
				ifTrue: [ ^ self ]
				ifFalse: [ #chapter ] ]
		ifFalse: [
			self class typeSymbols
				at: (self class typeSymbols 
					indexOf: self parent type
					ifAbsent: [ ^ self error: 'Interal Error' ]) + 1
				ifAbsent: [ self class typeSymbols last ] ])! !

!BOPortion methodsFor: 'printing' stamp: 'lr 12/20/2009 14:36'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' type: '; print: self type! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 00:35'!
type
	^ type! !

!BOPortion methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:34'!
type: aSymbol
	"Update the type of the receiver if necessary. Also update the children."
	
	(self type = aSymbol)
		ifTrue: [ ^ self ].
	(self class typeSymbols includes: aSymbol)
		ifFalse: [ self error: 'Invalid type: ' , aSymbol printString ].
	type := aSymbol.
	self enumerator
		do: [ :each | each parent: self ]! !

!BOPublication class methodsFor: 'testing' stamp: 'lr 5/26/2009 21:38'!
canBeParentOf: aStructureClass
	^ self childClasses anySatisfy: [ :class | aStructureClass includesBehavior: class ]! !

!BOPublication class methodsFor: 'accessing' stamp: 'lr 6/3/2009 21:50'!
childClasses
	^ Array with: BOPublication with: BONote with: BOFigure! !

!BOPublication class methodsFor: 'descriptions' stamp: 'lr 5/19/2009 15:32'!
descriptionIgnoredBody
	^ MABooleanDescription new
		checkboxLabel: 'Ignore body';
		accessor: #ignoredBody;
		default: false;
		priority: 162;
		beEditable;
		yourself! !

!BOPublication class methodsFor: 'descriptions' stamp: 'lr 5/19/2009 15:32'!
descriptionIgnoredTitle
	^ MABooleanDescription new
		checkboxLabel: 'Ignore title';
		accessor: #ignoredTitle;
		label: 'Export';
		default: false;
		priority: 160;
		beEditable;
		yourself! !

!BOPublication class methodsFor: 'descriptions' stamp: 'lr 11/16/2009 09:13'!
descriptionNumberedTitle
	^ MABooleanDescription new
		checkboxLabel: 'Numbered title';
		accessor: #numberedTitle;
		label: 'Display';
		default: true;
		priority: 150;
		beEditable;
		yourself
! !

!BOPublication class methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:18'!
typeLabels
	^ #('Book' 'Part' 'Chapter' 'Section' 'Subsection' 'Subsubsection' 'Paragraph')! !

!BOPublication class methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:18'!
typeSymbols
	^ #(book part chapter section subsection subsubsection paragraph)! !

!BOPublication class methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:22'!
typeSymbolsAndLabels
	^ ((1 to: self typeSymbols size) collect: [ :index |	(self typeSymbols at: index) -> (self typeLabels at: index) ])! !

!BOPublication methodsFor: 'visiting' stamp: 'lr 5/13/2009 11:30'!
accept: aVisitor
	aVisitor visitPublication: self! !

!BOPublication methodsFor: 'accessing-commands' stamp: 'lr 11/16/2009 08:07'!
defaultChildClass
	^ BOPortion! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:30'!
ignoredBody
	^ ignoredBody! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:30'!
ignoredBody: aBoolean
	ignoredBody := aBoolean! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:29'!
ignoredTitle
	^ ignoredTitle! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:29'!
ignoredTitle: aBoolean
	ignoredTitle := aBoolean! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:20'!
isBook
	^ self type = #book! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:20'!
isChapter
	^ self type = #chapter! !

!BOPublication methodsFor: 'testing' stamp: 'lr 5/19/2009 15:36'!
isIgnoredBody
	^ self ignoredBody ifNil: [ false ]! !

!BOPublication methodsFor: 'testing' stamp: 'lr 5/19/2009 15:30'!
isIgnoredTitle
	^ self ignoredTitle ifNil: [ false ]! !

!BOPublication methodsFor: 'testing' stamp: 'lr 5/19/2009 15:30'!
isNumberedTitle
	^ self numberedTitle ifNil: [ true ]! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 11:23'!
isParagraph
	^ self type = #paragraph! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:20'!
isPart
	^ self type = #part! !

!BOPublication methodsFor: 'testing' stamp: 'lr 4/23/2009 19:32'!
isPublication
	^ true! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:20'!
isSection
	^ self type = #section! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:20'!
isSubsection
	^ self type = #subsection! !

!BOPublication methodsFor: 'testing-type' stamp: 'lr 11/16/2009 00:21'!
isSubsubsection
	^ self type = #subsubsection! !

!BOPublication methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:05'!
label
	"Answer a label for the type of the receiver, like Book, Part, Chapter, Section, ..."

	^ self class typeLabels
		at: (self class typeSymbols
			indexOf: self type
			ifAbsent: [ String new ])
		ifAbsent: [ String new ]! !

!BOPublication methodsFor: 'accessing' stamp: 'lr 12/20/2009 11:57'!
number
	^ nil! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:29'!
numberedTitle
	^ numberedTitle! !

!BOPublication methodsFor: 'accessing-settings' stamp: 'lr 5/19/2009 15:29'!
numberedTitle: aBoolean
	numberedTitle := aBoolean! !

!BOPublication methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:06'!
type
	"Answer the type of the receiver, such as #book, #part, #chapter, #section, ..."

	self subclassResponsibility! !

BOPublication subclass: #BOSection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Deprecated'!

BOPublication subclass: #BOSubsection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Deprecated'!

PRInternalLink subclass: #BOReferenceLink
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Utilities'!

!BOReferenceLink class methodsFor: 'initialization' stamp: 'lr 6/19/2009 16:37'!
initialize
	| link |
	PRValueLink allInstances do: [ :each |
		each selector = #bookReference: ifTrue: [ 
			link := self reference: 'ref:' , (each
				parameterAt: 'target'
				ifAbsent: [ '.' ]).
			link owner: each owner.
			each becomeForward: link ] ]! !

!BOReferenceLink class methodsFor: 'testing' stamp: 'lr 6/19/2009 16:16'!
isValidReference: aString
	^ aString beginsWith: 'ref:'! !

!BOReferenceLink class methodsFor: 'accessing' stamp: 'lr 6/19/2009 16:16'!
priority
	^ 0! !

!BOReferenceLink class methodsFor: 'instance-creation' stamp: 'lr 6/19/2009 16:32'!
reference: aString
	^ super reference: (aString allButFirst: 4)! !

!BOReferenceLink methodsFor: 'visiting' stamp: 'lr 6/19/2009 16:15'!
accept: aVisitor
	aVisitor visitReferenceLink: self! !

!BOReferenceLink methodsFor: 'testing' stamp: 'lr 6/19/2009 16:27'!
isEmbedded
	^ false! !

!BOReferenceLink methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:45'!
title
	^ (self target notNil and: [ self target isFigure or: [ self target isPublication and: [ self target isBook not ] ] ])
		ifTrue: [ self target label ]
		ifFalse: [ super title ]! !

PRVisitor subclass: #BOIndexBuilder
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Utilities'!

!BOIndexBuilder methodsFor: 'visiting' stamp: 'lr 6/4/2009 09:39'!
start: anObject
	index := Dictionary new.
	super start: anObject.
	^ index! !

!BOIndexBuilder methodsFor: 'visiting-structure' stamp: 'lr 6/4/2009 09:38'!
visitChildren: aDecoration
	self visitAll: aDecoration children! !

!BOIndexBuilder methodsFor: 'visiting-document' stamp: 'lr 6/4/2009 09:43'!
visitIndexLink: aLink
	(index at: aLink entries first ifAbsentPut: [ OrderedCollection new ])
		addLast: aLink owner! !

!BOIndexBuilder methodsFor: 'visiting-structure' stamp: 'lr 6/4/2009 09:38'!
visitStructure: aStructure
	aStructure description do: [ :each |
		each isDocumentDescription
			ifTrue: [ self visit: (aStructure readUsing: each) ] ]! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 5/13/2009 11:16'!
visitBook: aStructure
	self visitPublication: aStructure! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 6/3/2009 21:53'!
visitFigure: anObject
	self visitFile: anObject! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 5/1/2009 22:11'!
visitIndexLink: aLink
	self visitLink: aLink! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 11/16/2009 08:08'!
visitPortion: aStructure
	self visitPublication: aStructure! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 5/13/2009 11:16'!
visitPublication: aStructure
	self visitCase: aStructure! !

!PRVisitor methodsFor: '*pier-book-model' stamp: 'lr 6/19/2009 16:22'!
visitReferenceLink: aLink
	self visitInternalLink: aLink! !

!PRLinkRenderer methodsFor: '*pier-book' stamp: 'lr 5/1/2009 22:35'!
visitIndexLink: aLink
	html anchor name: aLink title.
	parent visitAll: aLink children! !

Object subclass: #BOLatexStream
	instanceVariableNames: 'stream'
	classVariableNames: 'LatexCharacters'
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOLatexStream class methodsFor: 'initialization' stamp: 'lr 1/12/2010 19:45'!
initialize
	LatexCharacters := Dictionary new.
	LatexCharacters
		at: $# put: '\#';
		at: $& put: '\&';
		at: $% put: '\%';
		at: $$ put: '\$';
		at: ${ put: '\{';
		at: $} put: '\}';
		at: $_ put: '\_';
		at: $" put: '\symbol{34}';
		" -- "
		at: $[ put: '{[}';
		at: $] put: '{]}';
		at: $| put: '$|$';
		at: $< put: '$<$';
		at: $> put: '$>$';
		at: $/ put: '$/$';
		at: $\ put: '$\backslash$';
		at: $^ put: '$\uparrow$';
		" -- "
		at: $ put: '\"{a}';
		at: $ put: '\"{A}';
		at: $ put: '\"{e}';
		at: $ put: '\"{E}';
		at: $ put: '\"{i}';
		at: $ put: '\"{I}';
		at: $ put: '\"{o}';
		at: $ put: '\"{O}';
		at: $ put: '\"{u}';
		at: $ put: '\"{U}';
		at: $ put: '\"{y}';
		at: $ put: '\"{Y}';
		" -- "
		at: $ put: '\^{a}';
		at: $ put: '\^{A}';
		at: $ put: '\^{e}';
		at: $ put: '\^{E}';
		at: $ put: '\^{i}';
		at: $ put: '\^{I}';
		at: $ put: '\^{o}';
		at: $ put: '\^{O}';
		at: $ put: '\^{u}';
		at: $ put: '\^{U}';
		" -- "
		at: $ put: '\`{a}';
		at: $ put: '\`{A}';
		at: $ put: '\`{e}';
		at: $ put: '\`{E}';
		at: $ put: '\`{i}';
		at: $ put: '\`{I}';
		at: $ put: '\`{o}';
		at: $ put: '\`{O}';
		at: $ put: '\`{u}';
		at: $ put: '\`{U}';
		" -- "
		at: $ put: '\''{a}';
		at: $ put: '\''{A}';
		at: $ put: '\''{e}';
		at: $ put: '\''{i}';
		at: $ put: '\''{I}';
		at: $ put: '\''{o}';
		at: $ put: '\''{O}';
		at: $ put: '\''{u}';
		at: $ put: '\''{U}';
		" -- "
		at: $ put: '\c{c}';
		at: $ put: '\c{C}';
		" -- "
		at: $~ put: '\~{}';
		at: $ put: '\~{n}';
		at: $ put: '\~{N}'! !

!BOLatexStream class methodsFor: 'instance creation' stamp: 'lr 5/13/2009 11:21'!
on: aStream
	^ self basicNew initializeOn: aStream! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:53'!
comment: anObject
	stream nextPut: $%; space; nextPutAll: anObject asString; lf! !

!BOLatexStream methodsFor: 'accessing' stamp: 'lr 5/13/2009 11:24'!
contents
	^ stream contents! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:58'!
cr! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:53'!
curly: anObject
	stream nextPut: ${.
	self print: anObject.
	stream nextPut: $}! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:58'!
environment: aString do: anObject
	self tag: 'begin'; curly: aString; lf.
	self print: anObject.
	self tag: 'end'; curly: aString; lf! !

!BOLatexStream methodsFor: 'initialization' stamp: 'lr 5/13/2009 09:09'!
initializeOn: aStream
	stream := aStream! !

!BOLatexStream methodsFor: 'testing' stamp: 'lr 5/13/2009 11:23'!
isEmpty
	^ stream isEmpty! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 5/19/2009 15:52'!
lf
	(stream position isZero
		or: [ stream position > 1
		and: [ (stream contents at: stream position) = Character lf
		and: [ (stream contents at: stream position - 1) = Character lf ] ] ])
			ifTrue: [ ^ self ].
	stream lf! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 9/15/2009 08:56'!
lineBreak
	stream nextPutAll: '\\'
	! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 5/19/2009 15:04'!
nextPut: aCharacter
	| string |
	string := LatexCharacters at: aCharacter ifAbsent: nil.
	string isNil
		ifTrue: [ stream nextPut: aCharacter ]
		ifFalse: [ stream nextPutAll: string ]! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 5/13/2009 08:59'!
nextPutAll: anObject
	anObject asString do: [ :each | self nextPut: each ]! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 10/27/2009 08:35'!
print: anObject
	anObject isNil
		ifTrue: [ ^ self ].
	anObject isBlock
		ifTrue: [ anObject value ]
		ifFalse: [ self nextPutAll: anObject displayString ]! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 5/19/2009 15:06'!
space
	stream space! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:54'!
square: anObject
	stream nextPut: $[.
	self print: anObject.
	stream nextPut: $]! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 7/14/2009 15:11'!
star
	stream nextPut: $*! !

!BOLatexStream methodsFor: 'accessing' stamp: 'lr 5/13/2009 11:04'!
stream
	^ stream! !

!BOLatexStream methodsFor: 'streaming' stamp: 'lr 5/19/2009 15:06'!
tab
	stream tab! !

!BOLatexStream methodsFor: 'conveniance' stamp: 'lr 5/19/2009 15:54'!
tag: aString
	stream nextPut: $\; nextPutAll: aString! !

!PREmbeddedRenderer methodsFor: '*pier-book-model' stamp: 'lr 6/11/2009 09:34'!
visitFigure: aStructure
	html div class: 'figure'; with: [
		html anchor name: (aStructure name
			select: [ :each | each isAlphaNumeric ]).
		super visitFigure: aStructure.
		html label: [
			html span: [ html text: 'Figure '; render: aStructure number; text: ':' ].
			html text: ' '; text: aStructure title.
			(aStructure title endsWith: '.')
				ifFalse: [ html text: '.' ] ] ]! !

!WriteStream methodsFor: '*pier-book' stamp: 'lr 5/19/2009 16:00'!
lf
	self nextPut: Character lf! !

PRWidget subclass: #BONoteWidget
	instanceVariableNames: 'component command note'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Utilities'!

!BONoteWidget class methodsFor: 'testing' stamp: 'lr 5/3/2009 12:41'!
isAbstract
	^ false! !

!BONoteWidget class methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:41'!
label
	^ 'Book Notes'! !

!BONoteWidget methodsFor: 'private' stamp: 'lr 5/3/2009 12:55'!
buildCommand
	^ (self context
		structure: self note
		command: self note editCommandClass new)
			command! !

!BONoteWidget methodsFor: 'private' stamp: 'lr 5/3/2009 12:54'!
buildComponent
	^ (self buildDescription asComponentOn: self command)
		onAnswer: [ :result | self execute: result ];
		addValidatedForm;
		yourself! !

!BONoteWidget methodsFor: 'private' stamp: 'lr 5/3/2009 12:55'!
buildDescription
	^ self command description select: [ :each | each propertyAt: #public ifAbsent: [ false ] ]! !

!BONoteWidget methodsFor: 'private' stamp: 'lr 5/3/2009 12:56'!
buildNote
	^ BONote new
		title: self context structure title;
		yourself! !

!BONoteWidget methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:54'!
children
	^ Array with: self component! !

!BONoteWidget methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:55'!
command
	^ command ifNil: [ command := self buildCommand ]! !

!BONoteWidget methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:54'!
component
	^ component ifNil: [ component := self buildComponent ]! !

!BONoteWidget methodsFor: 'actions' stamp: 'lr 5/3/2009 13:02'!
execute: anEditCommand
	| addCommand |
	note := component := command := nil.
	anEditCommand ifNil: [ ^ self ].
	addCommand := PRAddCommand new.
	self context command: addCommand.
	addCommand
		name: (addCommand
			uniqueName: 'note'
			in: self context structure);
		type: BONote;
		beUnchecked;
		execute.
	self context
		structure: addCommand answer structure
		command: anEditCommand.
	anEditCommand
		beUnchecked;
		execute! !

!BONoteWidget methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:57'!
note
	^ note ifNil: [ note := self buildNote ]! !

!BONoteWidget methodsFor: 'accessing' stamp: 'lr 5/3/2009 12:40'!
notes
	^ (self context enumerator
		select: [ :each | each isNote ])
		asSortedCollection: [ :a :b | a timestamp > b timestamp ]! !

!BONoteWidget methodsFor: 'rendering-adding' stamp: 'lr 5/3/2009 17:09'!
renderAddOn: html
	html div class: 'addnote'; with: [
		html heading: 'Add a Note'.
		html render: self component ]! !

!BONoteWidget methodsFor: 'rendering' stamp: 'lr 5/19/2009 11:38'!
renderContentOn: html
	self context structure isPublication
		ifFalse: [ ^ self ].
	html div class: 'notes'; with: [
		self renderNotes: self notes on: html.
		self renderAddOn: html ]! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 12:25'!
renderNote: aNote on: html
	html div class: 'note'; with: [ 
		self renderNoteMeta: aNote on: html.
		self renderNoteBody: aNote on: html ]! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 17:18'!
renderNoteBody: aNote on: html
	| line |
	html div class: 'body'; with: [
		aNote note lines do: [ :string |
			line := string.
			html paragraph: [
				line isEmpty
					ifTrue: [ html space ]
					ifFalse: [
						[ line notEmpty and: [ line first = $ ] ]
							whileTrue: [ html space. line := line copyWithoutFirst ].
						html text: line ] ] ] ]! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 12:33'!
renderNoteMeta: aNote on: html 
	html div class: 'meta'; with: [
		self renderNoteMetaAuthor: aNote on: html.
		self renderNoteMetaPublication: aNote on: html.
		self renderNoteMetaCommands: aNote on: html ]! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 12:32'!
renderNoteMetaAuthor: aNote on: html.
	html span
		class: 'author';
		with: aNote author.
	html text: ' '! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 12:48'!
renderNoteMetaCommands: aNote on: html
	| commands contexts |
	commands := Array with: PREditCommand with: PRMoveCommand with: PRRemoveCommand.
	contexts := (commands
		collect: [ :class | self context structure: aNote command: class new ])
		select: [ :each | each isValid ].
	contexts isEmpty ifTrue: [ ^ self ].
	html span class: 'commands'; with: [
		contexts
			do: [ :each | html anchor goto: each; with: each command label ]
			separatedBy: [ html text: ', ' ] ].
	html text: ' '! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 13:10'!
renderNoteMetaPublication: aNote on: html 
	html span
		class: 'publication';
		with: '(' , aNote timestamp date printString , 
			', ' , aNote timestamp time printMinutes ,
			')'.
	html text: ' '! !

!BONoteWidget methodsFor: 'rendering-listing' stamp: 'lr 5/3/2009 17:15'!
renderNotes: aCollection on: html
	aCollection isEmpty
		ifTrue: [ ^ self ].
	html div class: 'listing'; with: [
		html heading: 'User Contributed Notes'.
		aCollection do: [ :each |
			self renderNote: each on: html ] ]! !

PRFile subclass: #BOFigure
	instanceVariableNames: 'numberedTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Structures'!

!BOFigure class methodsFor: 'testing' stamp: 'lr 6/3/2009 21:48'!
canBeChildOf: aStructureClass
	^ aStructureClass includesBehavior: BOPublication! !

!BOFigure class methodsFor: 'testing' stamp: 'lr 6/3/2009 23:42'!
canBeParentOf: aStructureClass
	^ false! !

!BOFigure class methodsFor: 'accessing-description' stamp: 'lr 6/3/2009 21:52'!
descriptionFile
	^ super descriptionFile
		addCondition: (MACondition selector: #isImage) labelled: 'Select an image';
		yourself! !

!BOFigure class methodsFor: 'descriptions' stamp: 'lr 6/4/2009 09:27'!
descriptionNumber
	^ MAStringDescription new
		parameterName: 'number';
		accessor: #number;
		beReadonly;
		yourself! !

!BOFigure class methodsFor: 'descriptions' stamp: 'lr 6/4/2009 09:27'!
descriptionNumberedTitle
	^ MABooleanDescription new
		checkboxLabel: 'Numbered title';
		accessor: #numberedTitle;
		label: 'Display';
		default: true;
		priority: 150;
		beEditable;
		yourself! !

!BOFigure class methodsFor: 'initialization' stamp: 'lr 6/3/2009 23:30'!
initialize
	PRFile allInstances do: [ :structure |
		(structure findAncestor: [ :each | each isPublication ]) notNil
			ifTrue: [ structure primitiveChangeClassTo: self basicNew ] ]! !

!BOFigure class methodsFor: 'accessing' stamp: 'lr 11/16/2009 00:11'!
label
	^ 'Book Figure'! !

!BOFigure methodsFor: 'visiting' stamp: 'lr 6/3/2009 21:50'!
accept: aVisitor
	aVisitor visitFigure: self! !

!BOFigure methodsFor: 'accessing' stamp: 'lr 6/3/2009 23:06'!
index
	"Answer the index number of this figure."

	| root siblings |
	self parent isNil
		ifTrue: [ ^ nil ].
	root := self parent
		findAncestor: [ :each | each isPublication and: [ each isBook ] ]
		ifAbsent: [ ^ nil ].
	siblings := root enumerator everything
		select: [ :each | each isFigure ].
	^ siblings indexOf: self ifAbsent: [ nil ]! !

!BOFigure methodsFor: 'testing' stamp: 'lr 6/3/2009 22:44'!
isFigure
	^ true! !

!BOFigure methodsFor: 'testing' stamp: 'lr 6/4/2009 09:28'!
isNumberedTitle
	^ self numberedTitle ifNil: [ true ]! !

!BOFigure methodsFor: 'accessing' stamp: 'lr 11/16/2009 11:45'!
label
	^ 'Figure ' , self number! !

!BOFigure methodsFor: 'accessing' stamp: 'lr 6/3/2009 23:07'!
number
	^ self index asString! !

!BOFigure methodsFor: 'accessing-settings' stamp: 'lr 6/4/2009 09:27'!
numberedTitle
	^ numberedTitle! !

!BOFigure methodsFor: 'accessing-settings' stamp: 'lr 6/4/2009 09:28'!
numberedTitle: aBoolean
	numberedTitle := aBoolean! !

PRDocumentWriter subclass: #BOLatexWriter
	instanceVariableNames: 'items latex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Book-Latex'!

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 1/16/2010 11:27'!
convertHtmlEntities: aString
	^ ((((aString 
		copyReplaceAll: '&lt;' with: '<')
		copyReplaceAll: '&gt;' with: '>')
		copyReplaceAll: '&amp;' with: '&')
		copyReplaceAll: '&larr;' with: '<-')
		copyReplaceAll: '&rarr;' with: '<-'
! !

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 7/14/2009 14:56'!
findItem: aBlock ifAbsent: anAbsentBlock
	items size to: 1 by: -1 do: [ :index |
		(aBlock value: (items at: index))
			ifTrue: [ ^ items at: index ] ].
	^ anAbsentBlock value! !

!BOLatexWriter methodsFor: 'generating' stamp: 'lr 9/15/2009 08:50'!
generateBookPdf: aBook
	latex comment: 'hypersetup'.
	latex tag: 'hypersetup'; curly: [
		latex lf; tab; print: 'pdftitle='; curly: aBook title; print: ','.
		latex lf; tab; print: 'pdfauthor='; curly: aBook authors ]; lf.
	latex lf.! !

!BOLatexWriter methodsFor: 'generating' stamp: 'lr 12/21/2009 17:33'!
generateBookTitle: aBook
	latex comment: 'title'.
	latex tag: 'label'; curly: (self structureLabelFor: aBook); lf.
	latex tag: 'title'; curly: aBook title; lf.
	aBook authors isNil ifFalse: [
		latex tag: 'author'; curly: [
			(aBook authors findTokens: ',')
				do: [ :each | latex nextPutAll: each trimBlanks ]
				separatedBy: [ latex tag: 'and'; lf ].
			aBook additionalAuthors isNil ifFalse: [
				latex stream nextPutAll: '\\[4ex]'; lf.
				latex nextPutAll: 'with '; nextPutAll: aBook additionalAuthors trimBlanks ] ] ].
	latex tag: 'date'; curly: Date today; lf.
	latex lf! !

!BOLatexWriter methodsFor: 'generating' stamp: 'lr 7/14/2009 15:13'!
generatePublication: aStructure type: aString
	latex lf; lf.
	latex comment: aStructure absolutePath.
	aStructure isIgnoredTitle
		ifTrue: [ ^ self ].
	latex
		tag: aString;
		print: (aStructure isNumberedTitle
			ifFalse: [ '*' ]);
		curly: aStructure title;
		lf.
	latex
		tag: 'label';
		curly: (self structureLabelFor: aStructure);
		lf; lf! !

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 7/14/2009 15:10'!
publication
	^ self 
		findItem: [ :each | (each isKindOf: PRStructure) and: [ each isPublication ] ]
		ifAbsent: [ ^ self ].! !

!BOLatexWriter methodsFor: 'accessing' stamp: 'lr 10/27/2009 08:18'!
stream: aStream
	super stream: aStream.
	latex := BOLatexStream on: aStream.
	items := OrderedCollection new: 1000! !

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 7/14/2009 15:09'!
stringLabelFor: aString
	^ aString asString select: [ :each | each isAlphaNumeric ]! !

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 7/14/2009 15:08'!
structureLabelFor: aStructure
	^ self structureLabelFor: aStructure with: nil! !

!BOLatexWriter methodsFor: 'queries' stamp: 'lr 7/14/2009 15:09'!
structureLabelFor: aStructure with: aString
	| parents |
	parents := aStructure parents asOrderedCollection.
	[ parents isEmpty or: [ parents first isPublication ] ]
		whileFalse: [ parents removeFirst ].
	parents := parents 
		collect: [ :each | self stringLabelFor: each name ].
	aString isNil
		ifFalse: [ parents addLast: (self stringLabelFor: aString) ].
	^ parents reduce: [ :a :b | a , ':' , b ]! !

!BOLatexWriter methodsFor: 'visiting' stamp: 'lr 7/14/2009 14:22'!
visit: anObject
	| result |
	items addLast: anObject.
	result := super visit: anObject.
	items removeLast.
	^ result! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:27'!
visitAnchor: anAnchor
	latex 
		tag: 'label'; 
		curly: [ stream nextPutAll: (self structureLabelFor: self publication with: anAnchor name) ];
		lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:27'!
visitAnnotatedParagraph: aParagraph
	latex
		environment: aParagraph annotation
		do: [ super visitAnnotatedParagraph: aParagraph ]! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitBoldFormat: anObject
	latex tag: 'textbf'; curly: [ super visitBoldFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-structure' stamp: 'lr 12/30/2009 13:46'!
visitBook: aStructure
	latex comment: 'This is an automatically generated file, do not modify manually!!'.
	latex comment: 'This book is published using Seaside, Magritte and the Pier book publishing engine.'.
	latex comment: TimeStamp now asString.
	latex lf.
	
	stream nextPutAll: aStructure latexHeader; lf.
	latex lf.
	
	self generateBookPdf: aStructure.
	self generateBookTitle: aStructure.
	
	latex comment: 'document'.
	latex tag: 'begin'; curly: 'document'; lf.
	latex lf.
	
	latex tag: 'frontmatter'; lf.
	latex tag: 'maketitle'; lf.
	latex tag: 'tableofcontents'; lf.
	latex tag: 'sloppy'; lf.
	latex lf.
	
	latex tag: 'mainmatter'; lf.
	latex lf.

	super visitBook: aStructure.
	
	latex lf.
	latex tag: 'backmatter'; lf.
	
	stream nextPutAll: aStructure latexFooter; lf.
	latex tag: 'end'; curly: 'document'; lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 12:29'!
visitDefinitionList: aDefinitionList
	latex tag: 'begin'; curly: 'list'; curly: nil; curly: nil; lf.
	super visitDefinitionList: aDefinitionList.
	latex tag: 'end'; curly: 'list'; lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 5/19/2009 15:57'!
visitEmptyParagraph: anObject
	latex lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:27'!
visitExternalLink: anExternalLink
	latex
		tag: 'href';
		curly: [ stream nextPutAll: anExternalLink url ];
		curly: [ super visitExternalLink: anExternalLink ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:26'!
visitHeader: aHeader
	| kind |
	kind := #('subsubsection' 'paragraph' 'subparagraph')
		at: (aHeader level max: 3).
	latex tag: kind; star; curly: aHeader text; lf.
	latex tag: 'label'; curly: [ stream nextPutAll: (self structureLabelFor: self publication with: aHeader hash) ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:26'!
visitIndexLink: anIndexLink
	latex tag: 'index'; curly: anIndexLink title! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:26'!
visitInternalLink: anInternalLink
	anInternalLink isBroken
		ifTrue: [ ^ latex tag: 'textcolor'; curly: 'red'; curly: [ super visitInternalLink: anInternalLink ] ].
	(anInternalLink isEmbedded and: [ anInternalLink target isFigure ]) ifTrue: [
		latex tag: 'begin'; curly: 'figure'; square: 'h!!tbp'; lf.
		latex tab; tag: 'begin'; curly: 'center'; lf.
		latex tab; tab; tag: 'includegraphics'; square: [ stream nextPutAll: 'width='; print: ((anInternalLink parameterAt: 'width' ifAbsent: [ 100 ]) asNumber / 100.0); nextPutAll: '\textwidth' ]; curly: [ stream nextPutAll: (anInternalLink target file directory fullNameFor: anInternalLink target file filename) ]; lf.
		latex tab; tab; tag: 'caption'; curly: [
			anInternalLink hasChildren
				ifTrue: [ super visitInternalLink: anInternalLink ]
				ifFalse: [ latex nextPutAll: anInternalLink target title ].
			latex tag: 'label'; curly: [ stream nextPutAll: (self structureLabelFor: anInternalLink target) ] ]; lf.
		latex tab; tag: 'end'; curly: 'center'; lf.
		^ latex tag: 'end'; curly: 'figure'; lf ].
	latex
		tag: 'hyperref';
		square: [ stream nextPutAll: (self structureLabelFor: anInternalLink target with: anInternalLink anchor) ];
		curly: [ 
			anInternalLink hasChildren
				ifTrue: [ super visitInternalLink: anInternalLink ]
				ifFalse: [ latex nextPutAll: anInternalLink target title ] ]! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitItalicFormat: anObject
	latex tag: 'textit'; curly: [ super visitItalicFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:25'!
visitLineBreak: aLineBreak
	latex lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:25'!
visitLink: aLink
	aLink hasChildren
		ifTrue: [ super visitLink: aLink ]
		ifFalse: [ latex print: aLink reference ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 5/19/2009 15:57'!
visitListItem: anObject
	latex tag: 'item'; space.
	super visitListItem: anObject.
	latex lf! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 1/16/2010 11:27'!
visitMonospaceFormat: anObject
	latex tag: 'ct'; curly: (self convertHtmlEntities: anObject text)! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:25'!
visitOrderedList: anOrderedList
	latex environment: 'enumerate' do: [ super visitOrderedList: anOrderedList ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:25'!
visitParagraph: aParagraph
	super visitParagraph: aParagraph.
	latex lf; lf! !

!BOLatexWriter methodsFor: 'visiting-structure' stamp: 'lr 11/16/2009 08:10'!
visitPortion: aStructure
	self generatePublication: aStructure type: aStructure type.
	super visitPortion: aStructure! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 13:28'!
visitPreformatted: aPreformatted
	latex environment: 'lstlisting' do: [
		aPreformatted children 
			do: [ :each | stream nextPutAll: (self convertHtmlEntities: each text); lf ] ]! !

!BOLatexWriter methodsFor: 'visiting-structure' stamp: 'lr 7/14/2009 14:35'!
visitPublication: aStructure
	aStructure isIgnoredBody
		ifFalse: [ self visit: aStructure document ].
	aStructure enumerator do: [ :structure |
		structure isPublication
			ifTrue: [ self visit: structure ] ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:25'!
visitReferenceLink: aReferenceLink
	aReferenceLink isBroken ifFalse: [
		(aReferenceLink target isFigure or: [ aReferenceLink target isPublication ])
			ifTrue: [ ^ latex tag: 'autoref'; curly: (self structureLabelFor: aReferenceLink target) ] ].
	^ super visitReferenceLink: aReferenceLink! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitStrikethroughFormat: anObject
	latex tag: 'sout'; curly: [ super visitStrikethroughFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitSubscriptFormat: anObject
	latex tag: 'raisebox'; curly: '-0.25em'; curly: [ super visitSubscriptFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitSuperscriptFormat: anObject
	latex tag: 'raisebox'; curly: '0.25em'; curly: [ super visitSuperscriptFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 12:32'!
visitTable: aTable
	latex tag: 'begin'; curly: 'tabularx'; curly: [ latex tag: 'textwidth' ].
	latex curly: (String new: aTable columnCount withAll: $X); lf.
	aTable children do: [ :row |
		self visit: row.
		(row children allSatisfy: [ :cell | cell isHeading ])
			ifTrue: [ stream nextPutAll: '\\ \hline'; lf ]
			ifFalse: [ stream nextPutAll: '\\'; lf ] ].
	latex tag: 'end'; curly: 'tabularx'! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:20'!
visitTableCell: aTableCell
	aTableCell isHeading
		ifFalse: [ super visitTableCell: aTableCell ]
		ifTrue: [ latex tag: 'textbf'; curly: [ super visitTableCell: aTableCell ] ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 5/19/2009 15:55'!
visitTableRow: aTableRow
	self visitAll: aTableRow children separatedBy: [ stream nextPut: $& ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 12:58'!
visitTermItem: aTermObject
	latex tag: 'item'; tag: 'hspace'; curly: [ latex stream nextPutAll: '-\labelwidth' ].
	latex tag: 'textbf'; curly: [ self visitAll: aTermObject children ]; lf! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:24'!
visitText: aText
	latex nextPutAll: aText text! !

!BOLatexWriter methodsFor: 'visiting-formatting' stamp: 'lr 5/13/2009 11:37'!
visitUnderlineFormat: anObject
	latex tag: 'uline'; curly: [ super visitUnderlineFormat: anObject ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:24'!
visitUnorderedList: anUnorderedList
	latex environment: 'itemize' do: [ super visitUnorderedList: anUnorderedList ]! !

!BOLatexWriter methodsFor: 'visiting-document' stamp: 'lr 12/20/2009 11:24'!
visitValueLink: aValueLink
	| value |
	value := aValueLink valueIn: PRCurrentContext value.
	value isNil
		ifTrue: [ ^ self visitLink: aValueLink ].
	(value isCollection and: [ value isString not ])
		ifTrue: [ ^ self visitAll: value ].
	(value isKindOf: PRDocumentItem)
		ifTrue: [ ^ self visit: value ].
	latex print: value! !

!PRValueLink methodsFor: '*pier-book' stamp: 'lr 11/16/2009 11:35'!
bookContentsIn: aContext
	<value: 'book.contents' comment: 'Display a table of contents.'>

	| result |
	^ self lookupStructure: aContext structure do: [ :root |
		result := OrderedCollection new.
		root enumerator everything do: [ :structure |
			(structure isPublication and: [ structure isBook not and: [ structure isParagraph not ] ]) ifTrue: [
				(result isEmpty or: [ structure isPart ])
					ifTrue: [ result add: PRDefinitionList new ].
				self 
					bookLabel: (structure number 
						ifNil: [ ' ' ])
					class: nil
					target: structure 
					to: result last ] ].
		result ]
! !

!PRValueLink methodsFor: '*pier-book' stamp: 'lr 8/28/2009 22:21'!
bookFiguresIn: aContext
	<value: 'book.figures' comment: 'Display a table of figures.'>

	| result |
	^ self lookupStructure: aContext structure do: [ :root |
		result := OrderedCollection new.
		root enumerator everything do: [ :structure |
			(result isEmpty or: [ structure isPublication and: [ structure isPart ] ])
				ifTrue: [ result add: PRDefinitionList new ].
			(structure isFigure and: [ structure isNumberedTitle ]) ifTrue: [
				result last
					add: (PRTermItem with: (PRText content: structure number));
					add: (PRDataItem with: (PRInternalLink new
						add: (PRText content: structure title);
						anchor: (structure name 
							select: [ :each | each isAlphaNumeric ]);
						target: structure parent;
						yourself)) ] ].
		result ]! !

!PRValueLink methodsFor: '*pier-book' stamp: 'lr 6/4/2009 09:45'!
bookIndexIn: aContext
	<value: 'book.index' comment: 'Display an index of keywords.'>

	| structs |
	^ self lookupStructure: aContext structure do: [ :root |
		| index |
		index := BOIndexBuilder new start: root.
		(index keys asSortedCollection: [ :a :b | a asLowercase < b asLowercase ])
			inject: PRDefinitionList new
			into: [ :list :word |
				structs := Set new.
				list add: (PRTermItem with: (PRText content: word)).
				(index at: word) do: [ :struct |
					(structs includes: struct) ifFalse: [ 
						structs add: struct.
						list add: (PRDataItem with: (PRInternalLink new
							anchor: word;
							target: struct;
							yourself)) ] ].
				list ] ]! !

!PRValueLink methodsFor: '*pier-book-private' stamp: 'lr 6/4/2009 09:57'!
bookLabel: aString class: aClassString target: aStructure to: aDefinitionList
	| number |
	^ aDefinitionList
		add: (PRTermItem with: (PRText content: (aStructure isNil
			ifTrue: [ '&nbsp;' ]
			ifFalse: [ aString ])));
		add: (PRDataItem with: (aStructure isNil
			ifTrue: [ PRText content: '&nbsp;' ]
			ifFalse: [
				number := aStructure number.
				number isNil 
					ifTrue: [ number := String new ]
					ifFalse: [ number := number , ' ' ].
				PRInternalLink new 
					parameterAt: 'title' put: number , aStructure title;
					parameterAt: 'class' put: aClassString;
					target: aStructure;
					yourself ]));
		yourself! !

!PRValueLink methodsFor: '*pier-book' stamp: 'lr 6/4/2009 09:52'!
bookNavigationIn: aContext
	<value: 'book.navigation' comment: 'Display a navigation structure for books.'>

	^ self lookupStructure: aContext structure do: [ :structure |
		| root structures index result |
		root := structure
			findAncestor: [ :each | each isPublication and: [ each isBook ] ]
			ifAbsent: [ ^ nil ].
		structures := (aContext enumeratorFor: root) everything
			select: [ :each | each isPublication ].
		index := structures indexOf: (structure
			findAncestor: [ :each | each isPublication ])
			ifAbsent: [ ^ nil ].
		result := PRDefinitionList new.
		self
			bookLabel: '&rarr;' class: 'navigation-next' target: (structures at: index + 1 ifAbsent: [ nil ]) to: result;
			bookLabel: '&larr;' class: 'navigation-prev' target: (structures at: index - 1 ifAbsent: [ nil ]) to: result ]! !

!PRValueLink methodsFor: '*pier-book' stamp: 'lr 6/19/2009 16:41'!
bookParentIn: aContext
	<value: 'book.parent' comment: 'Display the parent structure for books.'>

	^ self lookupStructure: aContext structure do: [ :structure |
		self
			bookLabel: '&sect;'
			class: 'navigation-parent'
			target: (structure parent isNil 
				ifFalse: [ structure parent findAncestor: [ :each | each isPublication ] ]) 
			to: PRDefinitionList new ]! !
BOPortion initialize!
BOReferenceLink initialize!
BOLatexStream initialize!
BOFigure initialize!
